@@ -1,7 +1,7 @@
 imbalanced.rfsrc <- function(formula, data, ntree = 3000,
-                 method = c("rfq", "brf", "standard"),
-                 block.size = NULL, perf.type = NULL, fast = FALSE,
-                 ratio = NULL,  ...)
+                             method = c("rfq", "brf", "standard"), splitrule = "auc",
+                             perf.type = NULL, block.size = NULL, fast = FALSE,
+                             ratio = NULL,  ...)
 {
   ## preliminary checks: all are fatal
   ## parse the formula to ensure this is a two-class problem
@@ -43,12 +43,13 @@ imbalanced.rfsrc <- function(formula, data, ntree = 3000,
   ## brf method
   ##----------------------------------------
   if (method == "brf") {
-    ## TBD TBD currently cannot handle missing values
+    ## TBD2 currently cannot handle missing values
     data <- na.omit(data)
     yvar <- data[, formulaPrelim$yvar.names]
     ## for legacy reasons we maintain swr here
-    ## TBD TBD consider swor
-    o <- rfsrc(formula, data, ntree = ntree, 
+    ## TBD2 consider swor
+    o <- rfsrc(formula, data, ntree = ntree,
+               splitrule = splitrule,
                perf.type = perf.type,
                block.size = block.size,
                case.wt = make.wt(yvar),
@@ -76,14 +77,15 @@ imbalanced.rfsrc <- function(formula, data, ntree = 3000,
     ## undersampling of the majority class if requested
     ##-------------------------------------------------
     if (!is.null(ratio)) {
-      ## TBD TBD currently cannot handle missing values
+      ## TBD2 currently cannot handle missing values
       data <- na.omit(data)
       yvar <- data[, formulaPrelim$yvar.names]
       samp <- make.imbalanced.sample(ntree = ntree, ratio = ratio, y = yvar)
       dots$bootstrap <- dots$samp <- NULL
       o <- do.call(rfsrc.grow, c(list(formula = formula, data = data, ntree = ntree,
-                                 rfq = rfq.flag, perf.type = perf.type, block.size = block.size,
-                                 samp = samp, bootstrap = "by.user"), dots))
+                                      rfq = rfq.flag, splitrule = splitrule,
+                                      perf.type = perf.type, block.size = block.size,
+                                      samp = samp, bootstrap = "by.user"), dots))
     }
     ##-----------------------------------------------------------
     ## proceed to standard and rfq analysis without undersampling
@@ -92,7 +94,8 @@ imbalanced.rfsrc <- function(formula, data, ntree = 3000,
     ##-----------------------------------------------------------
     else {
       o <- do.call(rfsrc.grow, c(list(formula = formula, data = data, ntree = ntree,
-                                 rfq = rfq.flag, perf.type = perf.type, block.size = block.size), dots))    
+                                      rfq = rfq.flag, splitrule = splitrule,
+                                      perf.type = perf.type, block.size = block.size), dots))    
     }
   }
   ## return the object
